@@ -692,18 +692,14 @@ scm_register_gtype_instance_sinkfunc (GType type, void (*sinkfunc) (gpointer))
     g_array_append_val (sink_funcs, sf);
 }
 
-/* returns a goops object of class (gtype->class [type of gtypeinstance]) */
+/* returns a goops object of class (gtype->class type). this function exists for
+ * gobject.c:scm_c_gtype_instance_instance_init. all other callers should use
+ * scm_c_gtype_instance_to_scm. */
 SCM
-scm_c_gtype_instance_to_scm (gpointer ginstance)
+scm_c_gtype_instance_to_scm_typed (gpointer ginstance, GType type)
 {
-    GType type;
     SCM instance_smob, class, object;
 
-    if (!ginstance)
-        return SCM_BOOL_F;
-
-    type = G_TYPE_FROM_INSTANCE (ginstance);
-
     object = scm_c_gtype_instance_get_cached_goops (ginstance);
     if (!scm_is_false (object))
         return object;
@@ -726,6 +722,16 @@ scm_c_gtype_instance_to_scm (gpointer ginstance)
     return object;
 }
 
+SCM
+scm_c_gtype_instance_to_scm (gpointer ginstance)
+{
+    if (!ginstance)
+        return SCM_BOOL_F;
+
+    return scm_c_gtype_instance_to_scm_typed
+        (ginstance, G_TYPE_FROM_INSTANCE (ginstance));
+}
+
 SCM
 scm_c_make_gtype_instance (gpointer ginstance)
 {
@@ -932,16 +938,6 @@ SCM_DEFINE (scm_sys_function_to_method_public,
 }
 #undef FUNC_NAME
 
-void guile_gobject_log_handler (const gchar *log_domain, GLogLevelFlags log_level,
-                                const gchar *message, gpointer user_data)
-{
-    scm_c_gruntime_error (NULL,
-                          "~A: ~A",
-                          SCM_LIST2 (scm_str2string (log_domain ?
-                                                     log_domain : "(application)"),
-                                     scm_str2string (message)));
-}
-
 
 
 /**********************************************************************
@@ -970,14 +966,6 @@ scm_init_gnome_gobject_types (void)
 #include "gtype.x"
 #endif
 
-    /* handle the application, GLib, and GLib-GObject domains by default */
-    g_log_set_handler (NULL, G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL,
-                       guile_gobject_log_handler, NULL);
-    g_log_set_handler ("GLib", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL,
-                       guile_gobject_log_handler, NULL);
-    g_log_set_handler ("GLib-GObject", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL,
-                       guile_gobject_log_handler, NULL);
-
     quark_type = g_quark_from_static_string ("%scm-gtype->type");
     quark_class = g_quark_from_static_string ("%scm-gtype->class");
     quark_guile_gtype_class = g_quark_from_static_string ("%scm-guile-gtype-class");
