@@ -31,12 +31,15 @@
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-11)
   #:use-module (srfi srfi-13)
-  
+  #:use-module (ice-9 pretty-print)
+
   #:use-module (gnome gw support g-wrap)
   #:use-module (g-wrap enumeration)
   #:use-module (g-wrap rti)
   #:use-module (g-wrap c-types)
   
+  #:use-module ((g-wrap util) #:select (call-with-output-file/cleanup
+                                        any-str->c-sym-str))
   #:use-module (gnome gobject utils)
   #:use-module (gnome gw support slib)
   
@@ -729,3 +732,80 @@
               #:wrap-func ,wrap-func
               #:unwrap-func ,unwrap-func)))
      (add-type! ws t)))
+
+;;; We override the generation of the scheme wrapper, because we want to
+;;; avoid listing all exports of the module. See the lengthy comment in
+;;; guile/g-wrap/guile-runtime.c in g-wrap for a rationale.
+(define-method (generate-wrapset (lang <symbol>)
+                                 (wrapset <gobject-wrapset-base>)
+                                 (basename <string>))
+  (define (register-generics-without-rti)
+    (fold-functions
+     (lambda (func rest)
+       (cond
+        ((and (not (uses-rti-for-function? wrapset func))
+              (generic-name func)
+              (> (argument-count func) 0)
+              (class-name (first (argument-types func))))
+         (cons `(%gw:procedure->method-public
+                 ,(name func)
+                 ;; Specializers
+                 ',(map (lambda (arg)
+                          (let ((typespec (typespec arg)))
+                            (and (not (memq 'unspecialized
+                                            (options typespec)))
+                                 (class-name (type typespec)))))
+                        (filter visible? (arguments func)))
+                 ',(generic-name func)
+                 ;; Required argument count
+                 ,(- (input-argument-count func)
+                     (optional-argument-count func))
+                 ;; Optional arguments?
+                 ,(not (zero? (optional-argument-count func))))
+               rest))
+        (else rest)))
+     '() wrapset))
+
+  ;; The next method is in (g-wrap guile), which will generate the scm
+  ;; file. We overwrite it afterwards.
+  (next-method)
+
+  (if (module wrapset)
+      (call-with-output-file/cleanup
+       (string-append basename ".scm")
+       (lambda (port)
+         (define (++ . args)
+           (apply string-append args))
+
+         (for-each
+          (lambda (x) (display x port) (newline port))
+          '(";; Generated by G-Wrap: an experimental Guile C API-wrapper engine."
+            ";; Customized by guile-gnome; see (gnome gw support g-wrap) for details."))
+         (for-each
+          (lambda (x) (pretty-print x port))
+          (list
+           `(define-module ,(module wrapset)
+              #:use-module (oop goops)
+              #:use-module (gnome gw support modules)
+              ,@(if (slot-ref wrapset 'shlib-abs?)
+                    '(#:use-module (g-wrap config))
+                    '()))
+
+           (let ((wrapset-name-c-sym (any-str->c-sym-str
+                                      (symbol->string (name wrapset)))))
+             `(dynamic-call ,(++ "gw_init_wrapset_" wrapset-name-c-sym)
+                            (dynamic-link
+                             ,(if (slot-ref wrapset 'shlib-abs?)
+                                  `(string-append
+                                    *g-wrap-shlib-dir*
+                                    ,(slot-ref wrapset 'shlib-path))
+                                  (slot-ref wrapset 'shlib-path)))))
+           
+           ;; This is what we avoid:
+           ;;   `(export ',@(module-exports wrapset))
+           ;; Instead we do this lovely hack:
+           `(export-all-lazy! ',(module-exports wrapset))
+           `(begin ,@(register-generics-without-rti))
+           `(if (defined? '%generics)
+                (module-use! (module-public-interface (current-module))
+                             %generics))))))))
